import os



def get_path_for_csv(combination_name):
    PATH = f"result/{combination_name}/"
    FILE_NAME = f"{combination_name}.csv"
    file_path = f"{PATH}{FILE_NAME}"
    return file_path

def get_path_for_plotting(combination_name:str,file_name: str) -> str:
    file_without_ext = os.path.splitext(file_name)[0]
    output_path = f"result/{combination_name}/{file_without_ext}/{file_name}.png"
    return output_path

def get_path_for_plotting2(file_name: str) -> str:
    file_without_ext = os.path.splitext(file_name)[0]
    output_path = f"result/{file_without_ext}/{file_name}.png"
    return output_path

from itertools import product

def generate_combinations(window_range=range(1, 6), gap_range=range(1, 6)):
    return [{"window_size": w, "min_gap": g} for w, g in product(window_range, gap_range)]